import { Loci } from '../../mol-model/loci';
import { StructureElement } from '../../mol-model/structure';
import { LociLabel, LociLabelProvider } from '../../mol-plugin-state/manager/loci-label';
import { TmDetChainListCache, TmDetDescriptorCache } from './prop';
import { createResidueListsPerChain, getChainAndResidueIds } from './tmdet-color-theme';
import { getResidue } from './types';

const siteLabels = [
    "Side1",
    "Side2",
    "TM alpha",
    "TM beta",
    "TM re-entrant loop",
    "Interfacial Helix",
    "Unknown localization",
    "Membrane Inside"
];
const DefaultResidueLabel = 6; // Unknown localization


export const TmDetLabelProvider: LociLabelProvider = {
    label: (loci: Loci): LociLabel => {
        let labelText = siteLabels[DefaultResidueLabel];

        if (loci.kind == 'element-loci') {
            const unit = loci.elements[0].unit;
            const pdbId = unit.model.entryId;

            const descriptor = TmDetDescriptorCache.get(pdbId);
            if (!descriptor) {
                return labelText;
            }

            let chainList =  TmDetChainListCache.get(pdbId);
            if (!chainList) {
                chainList = createResidueListsPerChain(descriptor.chains);
                TmDetChainListCache.set(pdbId, chainList);
            }

            const location = StructureElement.Loci.getFirstLocation(loci);
            const { chainId, residueId } = getChainAndResidueIds(location!);
            const residue = getResidue(chainList, chainId!, residueId!);
            if (residue) {
                labelText = siteLabels[residue?.siteId];
            }

        }
        return labelText;
    }
}
